use crate::{error::Error, response::PagedResponse};

pub trait Client {
    type Request;

    fn get(&self, url: url::Url) -> Result<Self::Request, Error>;
    fn fetch(
        &self,
        req: Self::Request,
    ) -> impl std::future::Future<Output = Result<PagedResponse, Error>> + Send;
}

impl Client for reqwest::Client {
    type Request = reqwest::Request;

    fn get(&self, url: url::Url) -> Result<Self::Request, Error> {
        reqwest::Client::get(self, url)
            .build()
            .map_err(|e| Error::Request(Box::new(e)))
    }

    async fn fetch(&self, req: Self::Request) -> Result<PagedResponse, Error> {
        let response = self
            .execute(req)
            .await
            .map_err(|e| Error::Response(Box::new(e)))?;

        let response = response
            .text()
            .await
            .map_err(|e| Error::Response(Box::new(e)))?;

        str::parse(&response)
    }
}
